@@ -23,7 +23,8 @@
 #include <KMessageBox>
 
 
-ConnectionManager::ConnectionManager(QObject* parent) : QObject(parent)
+ConnectionManager::ConnectionManager(QObject* parent)
+    : QObject(parent), m_overrideAutoReconnect (false)
 {
     connect(this, SIGNAL(requestReconnect(Server*)), this, SLOT(handleReconnect(Server*)));
 }
@@ -59,8 +60,10 @@ void ConnectionManager::connectTo(Konversation::ConnectionFlag flag, const QStri
         if (!channel.isEmpty())
         {
             Konversation::ChannelSettings channelSettings(channel);
+            Konversation::ChannelList cl;
+            cl << channelSettings;
 
-            settings.setInitialChannel(channelSettings);
+            settings.setOneShotChannelList(cl);
         }
     }
 
@@ -86,6 +89,39 @@ void ConnectionManager::connectTo(Konversation::ConnectionFlag flag, int serverG
     connectTo(flag, settings);
 }
 
+void ConnectionManager::connectTo(Konversation::ConnectionFlag flag, const QList<KUrl>& list)
+{
+    QMap<QString,Konversation::ChannelList> serverChannels;
+    QMap<QString,ConnectionSettings> serverConnections;
+
+    QList<KUrl>::ConstIterator it = list.constBegin();
+    QList<KUrl>::ConstIterator end = list.constEnd();
+    for (; it != end; ++it)
+    {
+        ConnectionSettings cs;
+        decodeIrcUrl(it->url(), cs);
+        kDebug() << cs.name() << " - "
+                 << cs.server().host() << cs.server().port() << cs.server().password()
+                 << " - " << (cs.serverGroup()?cs.serverGroup()->name():"");
+        QString sname = (cs.serverGroup()?
+                         cs.serverGroup()->name():
+                         (cs.server().host()+':'+cs.server().port()) );
+
+        serverChannels[sname] += cs.oneShotChannelList();
+        if (!serverChannels.contains(sname))
+        {
+            serverConnections[sname] = cs;
+        }
+    }
+
+    // Perform the connection
+    QMap<QString,Konversation::ChannelList>::ConstIterator s_i = serverChannels.constBegin();
+    for (; s_i != serverChannels.constEnd(); ++s_i) {
+        serverConnections[s_i.key()].setOneShotChannelList(s_i.value());
+        connectTo(flag, serverConnections[s_i.key()]);
+    }
+}
+
 void ConnectionManager::connectTo(Konversation::ConnectionFlag flag, ConnectionSettings& settings)
 {
     if (!settings.isValid()) return;
@@ -145,6 +181,8 @@ void ConnectionManager::handleConnectionStateChange(Server* server, Konversation
 
     if (state == Konversation::SSConnected)
     {
+        m_overrideAutoReconnect = false;
+
         if (!m_activeIdentities.contains(identityId))
         {
             m_activeIdentities.insert(identityId);
@@ -173,7 +211,7 @@ void ConnectionManager::handleConnectionStateChange(Server* server, Konversation
 
 void ConnectionManager::handleReconnect(Server* server)
 {
-    if (!Preferences::self()->autoReconnect()) return;
+    if (!Preferences::self()->autoReconnect() || m_overrideAutoReconnect) return;
 
     ConnectionSettings settings = server->getConnectionSettings();
 
@@ -235,6 +273,14 @@ void ConnectionManager::quitServers()
         it.value()->quitServer();
 }
 
+void ConnectionManager::reconnectServers()
+{
+    QMap<int, Server*>::ConstIterator it;
+
+    for (it = m_connectionList.constBegin(); it != m_connectionList.constEnd(); ++it)
+        it.value()->reconnect();
+}
+
 void ConnectionManager::decodeIrcUrl(const QString& url, ConnectionSettings& settings)
 {
     if (!url.startsWith(QLatin1String("irc://"))) return;
@@ -314,7 +360,12 @@ void ConnectionManager::decodeIrcUrl(const QString& url, ConnectionSettings& set
 
     // Assigning channel.
     if (!channelSettings.name().isEmpty())
-        settings.setInitialChannel(channelSettings);
+    {
+        Konversation::ChannelList cl;
+        cl << channelSettings;
+
+        settings.setOneShotChannelList(cl);
+    }
 }
 
 void ConnectionManager::decodeAddress(const QString& address, ConnectionSettings& settings,
@@ -478,18 +529,24 @@ bool ConnectionManager::reuseExistingConnection(ConnectionSettings& settings, bo
 
         if (!dupe->isConnected())
         {
-            if (!settings.initialChannel().name().isEmpty())
-                dupe->updateAutoJoin(settings.initialChannel());
+            if (!settings.oneShotChannelList().isEmpty())
+                dupe->updateAutoJoin(settings.oneShotChannelList());
 
             if (!dupe->isConnecting())
                 dupe->reconnect();
         }
         else
         {
-            if (!settings.initialChannel().name().isEmpty())
+            if (!settings.oneShotChannelList().isEmpty())
             {
-                dupe->sendJoinCommand(settings.initialChannel().name(),
-                                      settings.initialChannel().password());
+                Konversation::ChannelList::ConstIterator it = settings.oneShotChannelList().constBegin();
+                Konversation::ChannelList::ConstIterator itend = settings.oneShotChannelList().constEnd();
+
+                for ( ; it != itend; ++it )
+                {
+                    dupe->sendJoinCommand((*it).name(), (*it).password());
+                }
+                settings.clearOneShotChannelList();
             }
         }
     }
@@ -584,4 +641,24 @@ Server* ConnectionManager::getAnyServer()
     return 0;
 }
 
+void ConnectionManager::involuntaryQuitServers()
+{
+    m_overrideAutoReconnect = true;
+
+    QMap<int, Server*>::ConstIterator it;
+
+    for (it = m_connectionList.constBegin(); it != m_connectionList.constEnd(); ++it)
+        it.value()->involuntaryQuit();
+}
+
+void ConnectionManager::reconnectInvoluntary()
+{
+    m_overrideAutoReconnect = false;
+
+    QMap<int, Server*>::ConstIterator it;
+
+    for (it = m_connectionList.constBegin(); it != m_connectionList.constEnd(); ++it)
+        it.value()->reconnectInvoluntary();
+}
+
 #include "connectionmanager.moc"
